@@ -1,13 +1,10 @@
 /* generator.c
  * The main DSP signal generation functions plus some misc. functions
- * Jim Jackson Oct 96    Linux Version
+ * Jim Jackson     Linux Version
  */
 
 /*
- * Copyright (C) 1997 Jim Jackson                    jj@scs.leeds.ac.uk
- *                    School of Computer Studies,
- *                    The University of Leeds,
- *                    Leeds, LS2 9JT, UK
+ * Copyright (C) 1997-2008 Jim Jackson                    jj@franjam.org.uk
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,6 +24,10 @@
 
 /* History...................
  * 
+ * 11May08  Bug in mksweep with x10 or x100 options and wide ranges
+ *          caused int overflow problems and corrupt waveforms
+ * 18Jun98  altered to make it compile on systems without soundcard
+ *          or dsp support.
  * 18Mar97  Moved maketone() into here
  * 17Mar97  added a mksweep() function - but as a separate function
  *          is not available from the generate() function
@@ -66,15 +67,15 @@
  */
 
 #include <stdio.h>
-#include <strings.h>
+#include <string.h>
 #include <errno.h>
 #include <ctype.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <sys/soundcard.h>
 #include <math.h>
+#include "config.h"
 
 #ifndef PI
 # ifdef M_PI
@@ -102,7 +103,7 @@ int N;
    unsigned char *p;
    
    for (p=bf; N--; ) {
-      *p++=(unsigned char)(((int)(*ip++)>>8)+128);
+      *p++=(unsigned char)(((*ip++)>>8)+128);
    }
    return(0);
 }
@@ -181,13 +182,13 @@ int N,afmt;
    }
 }
 
-/* maketone(wf,fr,N,afmt)   generate a tone of frequency fr, samplerate N/sec,
- *   waveform wf, does a malloc to get a new tone buffer.
+/* maketone(wf,fr,N,G,afmt)   generate a tone of frequency fr, samplerate N/sec,
+ *   waveform wf, and digital Gain G, does a malloc to get a new tone buffer.
  */
 
-unsigned char *maketone(wf,fr,N,afmt)
+unsigned char *maketone(wf,fr,N,G,afmt)
 char *wf;
-int fr,N,afmt;
+int fr,N,G,afmt;
 
 {
    unsigned char *bf;
@@ -195,14 +196,14 @@ int fr,N,afmt;
    
    bn=N<<(afmt==AFMT_S16_LE);
    if ((bf=(unsigned char *)malloc(bn))==NULL) return(NULL);
-   n=generate((fr)?wf:"OFF",bf,bn,fr,100,N,-1,afmt);
+   n=generate((fr)?wf:"OFF",bf,bn,fr,G,N,-1,afmt);
    return((n)?bf:NULL);
 }
 
 /* mksweep(bf,bfn,FMbuf,wavbuf,Fmin,Fmax,N,afmt)
  *    create a swept waveform in bf, size bfn, of format afmt. 
  *    Sweeping waveform is in FMbuf - THIS is 16 bit samples!!!!
- *    Swept waveform is is in wavbuf
+ *    Swept waveform is in wavbuf
  *    the Freq range is Fmin to Fmax Hz.   Current Samplerate is N/sec
  */
 
@@ -214,26 +215,30 @@ char *wavbuf;
 int Fmin,Fmax;
 unsigned int N,afmt;
 {
-   int i,F,dF,x;
+   int i,D,F,dF,x;
    short int *vp,*wb,*fmb;
    unsigned char *p;
 
-   if (VERBOSE) {
-      printf("Generating sweep waveform from %d Hz to %d Hz\n",Fmin,Fmax);
-   }
    F=(Fmin+Fmax+1)/2;    /* calc centre freq */
    dF=Fmax-F;            /* scale factor     */
+   D=32767;              /* adj to prevent int overflow */
+   while( dF>32767 ) { dF=(dF+1)/2; D=(D+1)/2; }
+
+   if (VERBOSE) {
+      printf("Generating sweep waveform from %d Hz to %d Hz\n",Fmin,Fmax);
+      printf("Centre Freq. %d Hz, sweep scaling %d/%d\n",F,dF,D);
+   }
    fmb=(short int *)FMbuf;
    if (afmt==AFMT_S16_LE) {
       wb=(short int *)wavbuf;
       for (x=i=0,vp=(short int *)bf; i<N; i++) {
 	 *vp++=wb[x];
-	 x=(N+F+x+(((int)(*fmb++)*dF)/32767))%N;
+	 x=(N+F+x+(((int)(*fmb++)*dF)/D))%N;
       }
    } else if (afmt==AFMT_U8) {
       for (x=i=0,p=bf; i<N; i++ ) {
 	 *p++=wavbuf[x];
-	 x=(N+F+x+(((int)(*fmb++)*dF)/32767))%N;
+	 x=(N+F+x+(((int)(*fmb++)*dF)/D))%N;
       }
    } else {
       i=0;
@@ -379,7 +384,7 @@ int R,afmt;
 mksawtooth(bf,bfn,freq,A,N,R,afmt)
 char *bf;
 int bfn;
-unsigned freq,A,N;
+unsigned int freq,A,N;
 int R,afmt;
 {
    int i,n,nc,h,t,b,d;
@@ -407,7 +412,7 @@ int R,afmt;
    } else if (afmt==AFMT_S16_LE) {
       t=2*A*32767/100; 
       for (i=0,vp=(short int *)bf; i<n; i++) {
-	 b=((i*nc)%n-d)*t/n;
+	 b=((long long int)((i*nc)%n-d)*(long long int)t)/n;
 	 *vp++=(short int)((b>32767)?32767:((b<-32767)?-32767:b));
       }
    } else return(0);
@@ -453,7 +458,7 @@ int R,afmt;
       t=2*A*32767/100; 
       for (i=0,vp=(short int *)bf; i<n; i++) {
 	 b=(i*nc*2)%(2*n);
-	 b=(((b<=n)?b:(2*n-b))-d)*t/n;
+	 b=((long long int)(((b<=n)?b:(2*n-b))-d)*(long long int)t)/n;
 	 *vp++=(short int)((b>32767)?32767:((b<-32767)?-32767:b));
       }
    } else return(0);
@@ -593,6 +598,41 @@ int R,afmt;
 
 char **getWavNames()
 {
-   return(WavNAMES);
+  return(WavNAMES);
+}
+
+/* genAllWaveforms(Wfs,bufs,F,samplerate,afmt)   generate all possible
+ *    waveforms at freq F and setup (*Wfs)[] with waveform names and (*bufs)[]
+ *    with the actual samples.
+ */
+
+genAllWaveforms(Wfs,bufs,F,samplerate,afmt)
+char **Wfs[];
+char **bufs[];
+int F;
+int samplerate;
+int afmt;
+{
+  int i,st,n,N;
+   
+  if ((*Wfs)==NULL) *Wfs=WavNAMES;
+  else {
+    for (n=0; WavNAMES[n]!=NULL; n++) { (*Wfs)[n]=WavNAMES[n]; }
+    (*Wfs)[n]=NULL;
+  }
+  for (n=0; WavNAMES[n]!=NULL; n++) { }  /* count the no. of waveforms */
+  
+  if ((*bufs) == NULL) {
+    if ((*bufs=(char **)malloc((n+1)*sizeof(char *)))==NULL) return(ENOMEM);
+    for (i=0; i<=n; i++) { (*bufs)[i]=NULL; }
+  }
+  N=samplerate<<(afmt==16);
+  for (i=0; i<n ; i++) {
+    if ((*bufs)[i]!=NULL) free((*bufs)[i]);
+    if (((*bufs)[i]=(char *)malloc(N))==NULL) return(ENOMEM);
+    if ((st=generate((WavNAMES)[i],(*bufs)[i],N,F,100,samplerate,-1,afmt))==0) 
+	return(st);
+  }
+  return(0);
 }
 
