@@ -1,24 +1,27 @@
 /* sweepgen.h
  * header file for the Ncurses based sweep generator
- * Jim Jackson  <jj@scs.leeds.ac.uk>  Jan 97
+ * Jim Jackson  <jj@franjam.org.uk>
  */
 
-#define VERSION "sweepgen  Ver. 1.0 (Feb 98)   Ncurses based Digital Sweep Generator"
+#include "config.h"
 
-#define DEFAULT_BUFFERS 3
-#define DAC_FILE "/dev/dsp"
-#define SAMPLERATE 22050
-#define EFAIL -1
+#define VERSTR "%s  Ver. %s   Ncurses based Digital Sweep Generator"
 
-extern int vflg,dflg;
+extern int vflg,dflg,vikeys;
+extern char *sys;
 
+extern char dac[];			/* name of output device */
 extern int DAC;
 extern unsigned int samplerate;         /* Samples/sec        */
 extern unsigned int stereo;             /* stereo mono */
 extern unsigned int afmt;               /* format for DSP  */
+extern int Bufspersec;                  /* Number of Buffers per sec */
 extern int Nfragbufs;                   /* number of driver buffers */
 extern int fragsize;                    /* size of driver buffer fragments */
 extern int fragsamplesize;              /* size of fragments in samples */
+extern int resolution;                  /* 1 = 1Hz resolution, 10 = 0.1HZ,
+extern int LWn;                         /* number of specified loadable waveforms */
+extern char **LWaa;                     /* array of specifed loadable waveform
 
     /* Sweeping channel.... */
 extern char wf[32];                     /* waveform type */
