@@ -4,4 +4,4 @@
 
 tones=../tones
 
-randnote c4 250 100 | $tones -v -i - sin 100
+./randnote c4 250 100 | $tones -v -i - sin 100
