@@ -4,4 +4,4 @@
 
 tones=../tones
 
-randtone 500 250 100 | $tones -i - sin 100
+./randtone 500 250 100 | $tones -i - sin 100
